
package org.onis.secu3;

import org.onis.secu3.manager.R;

import android.content.Context;
import android.graphics.Bitmap;
import android.graphics.PorterDuff;
import android.graphics.BitmapFactory;
import android.graphics.Canvas;
import android.graphics.PixelFormat;
import android.graphics.RadialGradient;
import android.graphics.Shader;
import android.graphics.Color;
import android.graphics.LinearGradient;
import android.graphics.Paint;
import android.graphics.Paint.Align;
import android.graphics.Point;
import android.graphics.Shader;
import android.graphics.SweepGradient;
import android.util.AttributeSet;
import android.util.Log;
import android.view.SurfaceHolder;
import android.view.SurfaceView;

public class WaveformView extends SurfaceView implements SurfaceHolder.Callback{
	
	private static int rpm2view; // 
	
	private WaveformPlotThread plot_thread;

	public WaveformView(Context context, AttributeSet attrs){
		super(context, attrs);
		setZOrderOnTop(true);
		
		
		getHolder().addCallback(this);
		getHolder().setFormat(PixelFormat.TRANSPARENT);
		plot_thread = new WaveformPlotThread(getHolder(), this);
	}
	
	public void surfaceChanged(SurfaceHolder holder, int format, int width, int height){
		
	}
	
	public void surfaceCreated(SurfaceHolder holder){
		
		Log.i("test", plot_thread.getState().toString());

		if (plot_thread.getState() == Thread.State.TERMINATED)
	    {
			plot_thread = new WaveformPlotThread(getHolder(), this);
	    }
		plot_thread.setRunning(true);
		plot_thread.start();
		
	}
	
	public void surfaceDestroyed(SurfaceHolder holder){
		boolean retry = true;
		plot_thread.setRunning(false);
		while (retry){
			try{
				plot_thread.join();
				retry = false;
			}catch(InterruptedException e){
				
			}
		}
	}
	
	@Override
	public void onDraw(Canvas canvas){
		PlotPoints(canvas);
	}
	
	public void set_data(int rpm){
		
		plot_thread.setRunning(false);
		//    
		rpm2view=rpm;
		plot_thread.setRunning(true);
	}
	
	public void PlotPoints(Canvas canvas){

		
		// 
		TestTah(canvas, rpm2view);
		
		
		
	}
	
	
	void TestTah(Canvas canvas, int value){
		
		//     0-270
		int value_270=value*270/6000;
    	// 
       // synchronized (sfh) {
        	int r=100; // 
        	//  90     
        	if (canvas.getWidth()<=canvas.getHeight()){
        		r=(int)(canvas.getWidth()*0.45);
        	}
        	else{
        		r=(int)(canvas.getHeight()*0.45);
        	}
        	int cx,cy;
        	
        		//  
				Paint paintCircle = new Paint();
				paintCircle.setColor(Color.LTGRAY);
				paintCircle.setAntiAlias(true);  
				paintCircle.setStyle(Paint.Style.STROKE);  
				paintCircle.setStrokeWidth(3); 
				Paint paintCircleFill = new Paint();
				paintCircleFill.setColor(Color.WHITE);
				paintCircleFill.setAntiAlias(true);  
				paintCircleFill.setStyle(Paint.Style.FILL);  
				paintCircleFill.setStrokeWidth(3); 

				Paint paintLine = new Paint();
				paintLine.setColor(Color.RED);
				paintLine.setAntiAlias(true);  
				paintLine.setStyle(Paint.Style.STROKE);  
				paintLine.setStrokeWidth(5); 
				
				//paintLine.setShader(new SweepGradient(100, 250, Color.RED, Color.BLUE));
				Paint paintText = new Paint();
				paintText.setColor(Color.WHITE);
				paintText.setAntiAlias(true);  
				//paintText.setStyle(Paint.Style.STROKE);  
				paintText.setStrokeWidth(1); 
				paintText.setTextSize(r/3);
				paintText.setTextAlign(Align.CENTER);
				Paint paintTextSmall = new Paint();
				paintTextSmall.setColor(Color.WHITE);
				paintTextSmall.setAntiAlias(true);  
				//paintText.setStyle(Paint.Style.STROKE);  
				paintTextSmall.setStrokeWidth(1); 
				paintTextSmall.setTextSize(r/5);
				paintTextSmall.setTextAlign(Align.CENTER);
				Paint paintWWW = new Paint();
				paintWWW.setColor(Color.WHITE);
				paintWWW.setTextSize(r/7);
				paintWWW.setTextAlign(Align.CENTER);
				paintWWW.setUnderlineText(true);
				
				
				
	        int angle=0;
	        Point mPoint = new Point(0,0);
	        // 
	        //canvas.drawColor(Color.BLACK);
           // canvas.drawColor(Color.TRANSPARENT);
            canvas.drawColor(Color.TRANSPARENT, PorterDuff.Mode.CLEAR);
            
           // Bitmap bg = BitmapFactory.decodeResource(getResources(), R.drawable.backgroundpic);
            //canvas.drawBitmap(bg, 0, 0, null);

            // 
            cx=canvas.getWidth()/2;
            cy=canvas.getHeight()/2;
            paintLine.setShader(new RadialGradient(cx, cy, r, Color.WHITE,Color.RED, Shader.TileMode.MIRROR));

			//  
			canvas.drawCircle(cx, cy, r, paintCircle);
			//   
			canvas.drawCircle(cx, cy, r/20, paintCircleFill);

			//
			//     
			int ii=0;
			for(int i=0; i<=270; i+=45){
				mPoint = GetCoordPointOnCircle(cx,cy,r,i);
				int ox,oy; //    
				ox=mPoint.x;
				oy=mPoint.y;
				//      (  )
				mPoint = GetCoordPointOnCircle(cx,cy,(int)(r*0.9),i); 
				canvas.drawLine(ox, oy, mPoint.x, mPoint.y, paintText); // 
				mPoint = GetCoordPointOnCircle(cx,cy,(int)(r*0.8),i); 
				//  
				String text=String.valueOf(ii++);
				int textHeight = (int)paintTextSmall.measureText(text);
				canvas.drawText(text, mPoint.x, mPoint.y+textHeight/2, paintTextSmall);
				
			}
					
			//    
			canvas.drawText(String.valueOf("RPM"), cx, cy+r-4*r/6, paintTextSmall);
			canvas.drawText(String.valueOf(value), cx, cy+r-r/6, paintText);
			canvas.drawText(String.valueOf("www.secu-3.org"), cx, cy-r/6, paintWWW);
            //         
			angle=value_270;
			 mPoint = GetCoordPointOnCircle(cx,cy,r,angle);
			canvas.drawLine(cx, cy, mPoint.x, mPoint.y, paintLine); // 
        
    	
    }
	
	//      ,   
	//    0  360  
	private Point GetCoordPointOnCircle(int cx, int cy, int r, int angle){
		
    	int rx,ry,xs,ys,xf,yf;
    	double c,s;
		//      -  
		rx=cx-cx;
		ry=cy+r-cy;
		c=Math.cos(Math.PI/4);
		s=Math.sin(Math.PI/4);
		xs=(int)(cx+rx*c-ry*s);//  
		ys=(int)(cy+rx*s+ry*c);
		//        
		rx=xs-cx;
		ry=ys-cy;
		c=Math.cos(3*Math.PI/2*angle/270);
		s=Math.sin(3*Math.PI/2*angle/270);
		xf=(int)(cx+rx*c-ry*s); // 
		yf=(int)(cy+rx*s+ry*c);
		Point mPoint = new Point(xf,yf);

		return mPoint;
	}
		
		
	
}
